@@ -35,7 +35,7 @@ from ray._common.utils import (
     get_ray_address_file,
     get_system_memory,
 )
-from ray.core.generated.runtime_env_common_pb2 import (
+from ray.core.generated.runtime_environment_pb2 import (
     RuntimeEnvInfo as ProtoRuntimeEnvInfo,
 )
 
@@ -266,18 +266,46 @@ def set_omp_num_threads_if_unset() -> bool:
     return True
 
 
-def set_visible_accelerator_ids() -> None:
+def set_visible_accelerator_ids() -> Mapping[str, Optional[str]]:
     """Set (CUDA_VISIBLE_DEVICES, ONEAPI_DEVICE_SELECTOR, HIP_VISIBLE_DEVICES,
     NEURON_RT_VISIBLE_CORES, TPU_VISIBLE_CHIPS , HABANA_VISIBLE_MODULES ,...)
-    environment variables based on the accelerator runtime.
+    environment variables based on the accelerator runtime. Return the original
+    environment variables.
     """
+    from ray._private.ray_constants import env_bool
+
+    original_visible_accelerator_env_vars = {}
+    override_on_zero = env_bool(
+        ray._private.accelerators.RAY_ACCEL_ENV_VAR_OVERRIDE_ON_ZERO_ENV_VAR,
+        True,
+    )
     for resource_name, accelerator_ids in (
         ray.get_runtime_context().get_accelerator_ids().items()
     ):
+        # If no accelerator ids are set, skip overriding the environment variable.
+        if not override_on_zero and len(accelerator_ids) == 0:
+            continue
+        env_var = ray._private.accelerators.get_accelerator_manager_for_resource(
+            resource_name
+        ).get_visible_accelerator_ids_env_var()
+        original_visible_accelerator_env_vars[env_var] = os.environ.get(env_var, None)
         ray._private.accelerators.get_accelerator_manager_for_resource(
             resource_name
         ).set_current_process_visible_accelerator_ids(accelerator_ids)
 
+    return original_visible_accelerator_env_vars
+
+
+def reset_visible_accelerator_env_vars(
+    original_visible_accelerator_env_vars: Mapping[str, Optional[str]]
+) -> None:
+    """Reset the visible accelerator env vars to the original values."""
+    for env_var, env_value in original_visible_accelerator_env_vars.items():
+        if env_value is None:
+            os.environ.pop(env_var, None)
+        else:
+            os.environ[env_var] = env_value
+
 
 class Unbuffered(object):
     """There's no "built-in" solution to programatically disabling buffering of
@@ -342,9 +370,10 @@ def _get_docker_cpus(
     # See: https://bugs.openjdk.java.net/browse/JDK-8146115
     if os.path.exists(cpu_quota_file_name) and os.path.exists(cpu_period_file_name):
         try:
-            with open(cpu_quota_file_name, "r") as quota_file, open(
-                cpu_period_file_name, "r"
-            ) as period_file:
+            with (
+                open(cpu_quota_file_name, "r") as quota_file,
+                open(cpu_period_file_name, "r") as period_file,
+            ):
                 cpu_quota = float(quota_file.read()) / float(period_file.read())
         except Exception:
             logger.exception("Unexpected error calculating docker cpu quota.")
@@ -392,7 +421,8 @@ def _get_docker_cpus(
 
 def get_num_cpus(
     override_docker_cpu_warning: bool = ENV_DISABLE_DOCKER_CPU_WARNING,
-) -> int:
+    truncate: bool = True,
+) -> float:
     """
     Get the number of CPUs available on this node.
     Depending on the situation, use multiprocessing.cpu_count() or cgroups.
@@ -403,6 +433,7 @@ def get_num_cpus(
             RAY_DISABLE_DOCKER_CPU_WARNING. By default, whether or not to log
             the warning is determined by the env variable
             RAY_DISABLE_DOCKER_CPU_WARNING.
+        truncate: truncates the return value and drops the decimal part.
     """
     cpu_count = multiprocessing.cpu_count()
     if os.environ.get("RAY_USE_MULTIPROCESSING_CPU_COUNT"):
@@ -444,7 +475,8 @@ def get_num_cpus(
                     f"truncated from {docker_count} to "
                     f"{int(docker_count)}."
                 )
-            docker_count = int(docker_count)
+            if truncate:
+                docker_count = int(docker_count)
             cpu_count = docker_count
 
     except Exception:
@@ -918,28 +950,25 @@ def get_wheel_filename(
 
     architecture = architecture or platform.processor()
 
-    if py_version_str in ["311", "310", "39", "38"] and architecture == "arm64":
-        darwin_os_string = "macosx_12_0_arm64"
-    else:
-        darwin_os_string = "macosx_12_0_x86_64"
+    assert sys_platform in ["darwin", "linux", "win32"], sys_platform
 
-    if architecture == "aarch64":
-        linux_os_string = "manylinux2014_aarch64"
-    else:
-        linux_os_string = "manylinux2014_x86_64"
-
-    os_strings = {
-        "darwin": darwin_os_string,
-        "linux": linux_os_string,
-        "win32": "win_amd64",
-    }
-
-    assert sys_platform in os_strings, sys_platform
+    if sys_platform == "darwin":
+        if architecture == "x86_64":
+            os_string = "macosx_12_0_x86_64"
+        else:
+            os_string = "macosx_12_0_arm64"
+    elif sys_platform == "linux":
+        if architecture == "aarch64" or architecture == "arm64":
+            os_string = "manylinux2014_aarch64"
+        else:
+            os_string = "manylinux2014_x86_64"
+    elif sys_platform == "win32":
+        os_string = "win_amd64"
 
     wheel_filename = (
         f"ray-{ray_version}-cp{py_version_str}-"
         f"cp{py_version_str}{'m' if py_version_str in ['37'] else ''}"
-        f"-{os_strings[sys_platform]}.whl"
+        f"-{os_string}.whl"
     )
 
     return wheel_filename
@@ -1220,7 +1249,7 @@ def check_version_info(
     cluster_metadata,
     this_process_address,
     raise_on_mismatch=True,
-    python_version_match_level="patch",
+    python_version_match_level=None,
 ):
     """Check if the Python and Ray versions stored in GCS matches this process.
     Args:
@@ -1230,7 +1259,8 @@ def check_version_info(
         raise_on_mismatch: Raise an exception on True, log a warning otherwise.
         python_version_match_level: "minor" or "patch". To which python version level we
             try to match. Note if "minor" and the patch is different, we will still log
-            a warning.
+            a warning. Default value is `RAY_DEFAULT_PYTHON_VERSION_MATCH_LEVEL` if it
+            exists, otherwise "patch"
 
     Behavior:
         - We raise or log a warning, based on raise_on_mismatch, if:
@@ -1244,9 +1274,15 @@ def check_version_info(
             - Python patch versions do not match, AND
             - python_version_match_level == 'minor' AND
             - raise_on_mismatch == False.
+
     Raises:
         Exception: An exception is raised if there is a version mismatch.
     """
+    if python_version_match_level is None:
+        python_version_match_level = os.environ.get(
+            "RAY_DEFAULT_PYTHON_VERSION_MATCH_LEVEL", "patch"
+        )
+
     cluster_version_info = (
         cluster_metadata["ray_version"],
         cluster_metadata["python_version"],
