@@ -3,8 +3,10 @@
 from ray.dashboard.modules.metrics.dashboards.common import (
     DashboardConfig,
     Panel,
+    PanelTemplate,
     Row,
     Target,
+    TargetTemplate,
 )
 
 # When adding a new panels for an OpRuntimeMetric, follow this format:
@@ -448,7 +450,7 @@ BLOCK_GENERATION_TIME_PANEL = Panel(
     id=8,
     title="Block Generation Time",
     description="Time spent generating blocks in tasks.",
-    unit="seconds",
+    unit="s",
     targets=[
         Target(
             expr='sum(ray_data_block_generation_time{{{global_filters}, operator=~"$Operator"}}) by (dataset, operator)',
@@ -463,7 +465,7 @@ TASK_SUBMISSION_BACKPRESSURE_PANEL = Panel(
     id=37,
     title="Task Submission Backpressure Time",
     description="Time spent in task submission backpressure.",
-    unit="seconds",
+    unit="s",
     targets=[
         Target(
             expr='sum(ray_data_task_submission_backpressure_time{{{global_filters}, operator=~"$Operator"}}) by (dataset, operator)',
@@ -477,24 +479,77 @@ TASK_SUBMISSION_BACKPRESSURE_PANEL = Panel(
 # Task Completion Time Percentiles
 TASK_COMPLETION_TIME_PANEL = Panel(
     id=38,
-    title="Task Completion Time",
-    description="Time spent running tasks to completion w/ backpressure.",
-    unit="seconds",
+    title="Task Completion Time Histogram (s)",
+    description="Time (in seconds) spent (including backpressure) running tasks to completion. Larger bars means more tasks finished within that duration range.",
     targets=[
         Target(
-            expr='increase(ray_data_task_completion_time{{{global_filters}, operator=~"$Operator"}}[5m]) / increase(ray_data_num_tasks_finished{{{global_filters}, operator=~"$Operator"}}[5m])',
-            legend="Task Completion Time: {{dataset}}, {{operator}}",
+            expr='sum by (le) (max_over_time(ray_data_task_completion_time_bucket{{{global_filters}, operator=~"$Operator", le!="+Inf"}}[$__range]))',
+            legend="{{le}} s",
+            template=TargetTemplate.HISTOGRAM_BAR_CHART,
         ),
     ],
+    unit="short",
     fill=0,
     stack=False,
+    template=PanelTemplate.BAR_CHART,
+)
+
+BLOCK_COMPLETION_TIME_PANEL = Panel(
+    id=61,
+    title="Block Completion Time Histogram (s)",
+    description="Time (in seconds) spent processing blocks to completion. If multiple blocks are generated per task, this is approximated by assuming each block took an equal amount of time to process. Larger bars means more blocks finished within that duration range.",
+    targets=[
+        Target(
+            expr='sum by (le) (max_over_time(ray_data_block_completion_time_bucket{{{global_filters}, operator=~"$Operator", le!="+Inf"}}[$__range]))',
+            legend="{{le}} s",
+            template=TargetTemplate.HISTOGRAM_BAR_CHART,
+        ),
+    ],
+    unit="short",
+    fill=0,
+    stack=False,
+    template=PanelTemplate.BAR_CHART,
+)
+
+BLOCK_SIZE_BYTES_PANEL = Panel(
+    id=62,
+    title="Block Size (Bytes) Histogram",
+    description="Size (in bytes) per block. Larger bars means more blocks are within that size range.",
+    targets=[
+        Target(
+            expr='sum by (le) (max_over_time(ray_data_block_size_bytes_bucket{{{global_filters}, operator=~"$Operator", le!="+Inf"}}[$__range]))',
+            legend="{{le}} bytes",
+            template=TargetTemplate.HISTOGRAM_BAR_CHART,
+        ),
+    ],
+    unit="short",
+    fill=0,
+    stack=False,
+    template=PanelTemplate.BAR_CHART,
+)
+
+BLOCK_SIZE_ROWS_PANEL = Panel(
+    id=63,
+    title="Block Size (Rows) Histogram",
+    description="Number of rows per block. Larger bars means more blocks are within that number of rows range.",
+    targets=[
+        Target(
+            expr='sum by (le) (max_over_time(ray_data_block_size_rows_bucket{{{global_filters}, operator=~"$Operator", le!="+Inf"}}[$__range]))',
+            legend="{{le}} rows",
+            template=TargetTemplate.HISTOGRAM_BAR_CHART,
+        ),
+    ],
+    unit="short",
+    fill=0,
+    stack=False,
+    template=PanelTemplate.BAR_CHART,
 )
 
 TASK_OUTPUT_BACKPRESSURE_TIME_PANEL = Panel(
     id=39,
     title="Task Output Backpressure Time",
     description="Time spent in output backpressure.",
-    unit="seconds",
+    unit="s",
     targets=[
         Target(
             expr='increase(ray_data_task_output_backpressure_time{{{global_filters}, operator=~"$Operator"}}[5m]) / increase(ray_data_num_tasks_finished{{{global_filters}, operator=~"$Operator"}}[5m])',
@@ -509,7 +564,7 @@ TASK_COMPLETION_TIME_WITHOUT_BACKPRESSURE_PANEL = Panel(
     id=40,
     title="Task Completion Time Without Backpressure",
     description="Time spent running tasks to completion w/o backpressure.",
-    unit="seconds",
+    unit="s",
     targets=[
         Target(
             expr='increase(ray_data_task_completion_time_without_backpressure{{{global_filters}, operator=~"$Operator"}}[5m]) / increase(ray_data_num_tasks_finished{{{global_filters}, operator=~"$Operator"}}[5m])',
@@ -523,7 +578,7 @@ TASK_COMPLETION_TIME_WITHOUT_BACKPRESSURE_PANEL = Panel(
 # Ray Data Metrics (Object Store Memory)
 INTERNAL_INQUEUE_BLOCKS_PANEL = Panel(
     id=13,
-    title="Operator Internal Inqueue Size (Blocks)",
+    title="Operator Internal Input Queue Size (Blocks)",
     description="Number of blocks in operator's internal input queue",
     unit="blocks",
     targets=[
@@ -538,7 +593,7 @@ INTERNAL_INQUEUE_BLOCKS_PANEL = Panel(
 
 INTERNAL_INQUEUE_BYTES_PANEL = Panel(
     id=14,
-    title="Operator Internal Inqueue Size (Bytes)",
+    title="Operator Internal Input Queue Size (Bytes)",
     description="Byte size of input blocks in the operator's internal input queue.",
     unit="bytes",
     targets=[
@@ -553,7 +608,7 @@ INTERNAL_INQUEUE_BYTES_PANEL = Panel(
 
 INTERNAL_OUTQUEUE_BLOCKS_PANEL = Panel(
     id=15,
-    title="Operator Internal Outqueue Size (Blocks)",
+    title="Operator Internal Output Queue Size (Blocks)",
     description="Number of blocks in operator's internal output queue",
     unit="blocks",
     targets=[
@@ -568,7 +623,7 @@ INTERNAL_OUTQUEUE_BLOCKS_PANEL = Panel(
 
 INTERNAL_OUTQUEUE_BYTES_PANEL = Panel(
     id=16,
-    title="Operator Internal Outqueue Size (Bytes)",
+    title="Operator Internal Output Queue Size (Bytes)",
     description=("Byte size of output blocks in the operator's internal output queue."),
     unit="bytes",
     targets=[
@@ -583,7 +638,7 @@ INTERNAL_OUTQUEUE_BYTES_PANEL = Panel(
 
 EXTERNAL_INQUEUE_BLOCKS_PANEL = Panel(
     id=2,
-    title="Operator External InQueue Size (Blocks)",
+    title="Operator External Input Queue Size (Blocks)",
     description="Number of blocks in operator's external input queue",
     unit="blocks",
     targets=[
@@ -598,7 +653,7 @@ EXTERNAL_INQUEUE_BLOCKS_PANEL = Panel(
 
 EXTERNAL_INQUEUE_BYTES_PANEL = Panel(
     id=27,
-    title="Operator External InQueue Size (bytes)",
+    title="Operator External Input Queue Size (bytes)",
     description="Byte size of blocks in operator's external input queue",
     unit="bytes",
     targets=[
@@ -611,10 +666,40 @@ EXTERNAL_INQUEUE_BYTES_PANEL = Panel(
     stack=False,
 )
 
-# Combined Inqueue and Outqueue Blocks Panel
-COMBINED_INQUEUE_OUTQUEUE_BLOCKS_PANEL = Panel(
+EXTERNAL_OUTQUEUE_BLOCKS_PANEL = Panel(
+    id=58,
+    title="Operator External Output Queue Size (Blocks)",
+    description="Number of blocks in operator's external output queue",
+    unit="blocks",
+    targets=[
+        Target(
+            expr='sum(ray_data_num_external_outqueue_blocks{{{global_filters}, operator=~"$Operator"}}) by (dataset, operator)',
+            legend="Number of Blocks: {{dataset}}, {{operator}}",
+        )
+    ],
+    fill=0,
+    stack=False,
+)
+
+EXTERNAL_OUTQUEUE_BYTES_PANEL = Panel(
+    id=59,
+    title="Operator External Output Queue Size (bytes)",
+    description="Byte size of blocks in operator's external output queue",
+    unit="bytes",
+    targets=[
+        Target(
+            expr='sum(ray_data_num_external_outqueue_bytes{{{global_filters}, operator=~"$Operator"}}) by (dataset, operator)',
+            legend="Number of Bytes: {{dataset}}, {{operator}}",
+        )
+    ],
+    fill=0,
+    stack=False,
+)
+
+# Combined Input Queue and Output Queue Blocks Panel
+COMBINED_INQUEUE_BLOCKS_PANEL = Panel(
     id=56,
-    title="Operator Combined Internal + External Inqueue Size (Blocks)",
+    title="Operator Combined Internal + External Input Queue Size (Blocks)",
     description="Total number of blocks in operator's internal + external input queue.",
     unit="blocks",
     targets=[
@@ -627,6 +712,21 @@ COMBINED_INQUEUE_OUTQUEUE_BLOCKS_PANEL = Panel(
     stack=False,
 )
 
+COMBINED_OUTQUEUE_BLOCKS_PANEL = Panel(
+    id=60,
+    title="Operator Combined Internal + External Output Queue Size (Blocks)",
+    description="Total number of blocks in operator's internal + external output queue.",
+    unit="blocks",
+    targets=[
+        Target(
+            expr='sum(ray_data_obj_store_mem_internal_outqueue_blocks{{{global_filters}, operator=~"$Operator"}} + ray_data_num_external_outqueue_blocks{{{global_filters}, operator=~"$Operator"}}) by (dataset, operator)',
+            legend="Combined Blocks: {{dataset}}, {{operator}}",
+        )
+    ],
+    fill=0,
+    stack=False,
+)
+
 PENDING_TASK_INPUTS_PANEL = Panel(
     id=34,
     title="Size of Blocks used in Pending Tasks (Bytes)",
@@ -677,7 +777,7 @@ ITERATION_INITIALIZATION_PANEL = Panel(
     id=12,
     title="Iteration Initialization Time",
     description="Seconds spent in iterator initialization code",
-    unit="seconds",
+    unit="s",
     targets=[
         Target(
             expr="sum(ray_data_iter_initialize_seconds{{{global_filters}}}) by (dataset)",
@@ -692,7 +792,7 @@ ITERATION_BLOCKED_PANEL = Panel(
     id=9,
     title="Iteration Blocked Time",
     description="Seconds user thread is blocked by iter_batches()",
-    unit="seconds",
+    unit="s",
     targets=[
         Target(
             expr="sum(ray_data_iter_total_blocked_seconds{{{global_filters}}}) by (dataset)",
@@ -707,7 +807,7 @@ ITERATION_USER_PANEL = Panel(
     id=10,
     title="Iteration User Time",
     description="Seconds spent in user code",
-    unit="seconds",
+    unit="s",
     targets=[
         Target(
             expr="sum(ray_data_iter_user_seconds{{{global_filters}}}) by (dataset)",
@@ -718,12 +818,132 @@ ITERATION_USER_PANEL = Panel(
     stack=False,
 )
 
+ITERATION_GET_PANEL = Panel(
+    id=70,
+    title="Iteration Get Time",
+    description="Seconds spent in ray.get() while resolving block references",
+    unit="seconds",
+    targets=[
+        Target(
+            expr="sum(ray_data_iter_get_seconds{{{global_filters}}}) by (dataset)",
+            legend="Seconds: {{dataset}}",
+        )
+    ],
+    fill=0,
+    stack=False,
+)
+
+ITERATION_NEXT_BATCH_PANEL = Panel(
+    id=71,
+    title="Iteration Next Batch Time",
+    description="Seconds spent getting the next batch from the block buffer",
+    unit="seconds",
+    targets=[
+        Target(
+            expr="sum(ray_data_iter_next_batch_seconds{{{global_filters}}}) by (dataset)",
+            legend="Seconds: {{dataset}}",
+        )
+    ],
+    fill=0,
+    stack=False,
+)
+
+ITERATION_FORMAT_BATCH_PANEL = Panel(
+    id=72,
+    title="Iteration Format Batch Time",
+    description="Seconds spent formatting the batch",
+    unit="seconds",
+    targets=[
+        Target(
+            expr="sum(ray_data_iter_format_batch_seconds{{{global_filters}}}) by (dataset)",
+            legend="Seconds: {{dataset}}",
+        )
+    ],
+    fill=0,
+    stack=False,
+)
+
+ITERATION_COLLATE_BATCH_PANEL = Panel(
+    id=73,
+    title="Iteration Collate Batch Time",
+    description="Seconds spent collating the batch",
+    unit="seconds",
+    targets=[
+        Target(
+            expr="sum(ray_data_iter_collate_batch_seconds{{{global_filters}}}) by (dataset)",
+            legend="Seconds: {{dataset}}",
+        )
+    ],
+    fill=0,
+    stack=False,
+)
+
+ITERATION_FINALIZE_BATCH_PANEL = Panel(
+    id=74,
+    title="Iteration Finalize Batch Time",
+    description="Seconds spent finalizing the batch",
+    unit="seconds",
+    targets=[
+        Target(
+            expr="sum(ray_data_iter_finalize_batch_seconds{{{global_filters}}}) by (dataset)",
+            legend="Seconds: {{dataset}}",
+        )
+    ],
+    fill=0,
+    stack=False,
+)
+
+ITERATION_BLOCKS_LOCAL_PANEL = Panel(
+    id=75,
+    title="Iteration Blocks Local",
+    description="Number of blocks already on the local node",
+    unit="blocks",
+    targets=[
+        Target(
+            expr="sum(ray_data_iter_blocks_local{{{global_filters}}}) by (dataset)",
+            legend="Blocks: {{dataset}}",
+        )
+    ],
+    fill=0,
+    stack=False,
+)
+
+ITERATION_BLOCKS_REMOTE_PANEL = Panel(
+    id=76,
+    title="Iteration Blocks Remote",
+    description="Number of blocks that require fetching from another node",
+    unit="blocks",
+    targets=[
+        Target(
+            expr="sum(ray_data_iter_blocks_remote{{{global_filters}}}) by (dataset)",
+            legend="Blocks: {{dataset}}",
+        )
+    ],
+    fill=0,
+    stack=False,
+)
+
+ITERATION_BLOCKS_UNKNOWN_LOCATION_PANEL = Panel(
+    id=77,
+    title="Iteration Blocks Unknown Location",
+    description="Number of blocks that have unknown locations",
+    unit="blocks",
+    targets=[
+        Target(
+            expr="sum(ray_data_iter_unknown_location{{{global_filters}}}) by (dataset)",
+            legend="Blocks: {{dataset}}",
+        )
+    ],
+    fill=0,
+    stack=False,
+)
+
 # Ray Data Metrics (Miscellaneous)
 SCHEDULING_LOOP_DURATION_PANEL = Panel(
     id=47,
     title="Scheduling Loop Duration",
     description=("Duration of the scheduling loop in seconds."),
-    unit="seconds",
+    unit="s",
     targets=[
         Target(
             expr="sum(ray_data_sched_loop_duration_s{{{global_filters}}}) by (dataset)",
@@ -831,7 +1051,11 @@ ALL_RESOURCES_UTILIZATION_PANEL = Panel(
     stack=False,
 )
 
-OPERATOR_PANELS = [ROWS_OUTPUT_PER_SECOND_PANEL, ALL_RESOURCES_UTILIZATION_PANEL]
+OPERATOR_PANELS = [
+    ROWS_OUTPUT_PER_SECOND_PANEL,
+    ALL_RESOURCES_UTILIZATION_PANEL,
+    COMBINED_INQUEUE_BLOCKS_PANEL,
+]
 
 DATA_GRAFANA_ROWS = [
     # Overview Row
@@ -844,7 +1068,8 @@ DATA_GRAFANA_ROWS = [
             ROWS_GENERATED_PANEL,
             OBJECT_STORE_MEMORY_PANEL,
             RUNNING_TASKS_PANEL,
-            COMBINED_INQUEUE_OUTQUEUE_BLOCKS_PANEL,
+            COMBINED_INQUEUE_BLOCKS_PANEL,
+            COMBINED_OUTQUEUE_BLOCKS_PANEL,
         ],
         collapsed=False,
     ),
@@ -881,6 +1106,8 @@ DATA_GRAFANA_ROWS = [
         panels=[
             INTERNAL_OUTQUEUE_BLOCKS_PANEL,
             INTERNAL_OUTQUEUE_BYTES_PANEL,
+            EXTERNAL_OUTQUEUE_BLOCKS_PANEL,
+            EXTERNAL_OUTQUEUE_BYTES_PANEL,
             MAX_BYTES_TO_READ_PANEL,
         ],
         collapsed=True,
@@ -890,6 +1117,8 @@ DATA_GRAFANA_ROWS = [
         title="Outputs",
         id=103,
         panels=[
+            BLOCK_SIZE_BYTES_PANEL,
+            BLOCK_SIZE_ROWS_PANEL,
             OUTPUT_BLOCKS_TAKEN_PANEL,
             OUTPUT_BYTES_TAKEN_PANEL,
             OUTPUT_BYTES_BY_NODE_PANEL,
@@ -908,6 +1137,7 @@ DATA_GRAFANA_ROWS = [
         id=104,
         panels=[
             TASK_COMPLETION_TIME_PANEL,
+            BLOCK_COMPLETION_TIME_PANEL,
             TASK_COMPLETION_TIME_WITHOUT_BACKPRESSURE_PANEL,
             TASK_OUTPUT_BACKPRESSURE_TIME_PANEL,
             TASK_SUBMISSION_BACKPRESSURE_PANEL,
@@ -954,6 +1184,14 @@ DATA_GRAFANA_ROWS = [
             ITERATION_INITIALIZATION_PANEL,
             ITERATION_BLOCKED_PANEL,
             ITERATION_USER_PANEL,
+            ITERATION_GET_PANEL,
+            ITERATION_NEXT_BATCH_PANEL,
+            ITERATION_FORMAT_BATCH_PANEL,
+            ITERATION_COLLATE_BATCH_PANEL,
+            ITERATION_FINALIZE_BATCH_PANEL,
+            ITERATION_BLOCKS_LOCAL_PANEL,
+            ITERATION_BLOCKS_REMOTE_PANEL,
+            ITERATION_BLOCKS_UNKNOWN_LOCATION_PANEL,
         ],
         collapsed=True,
     ),
@@ -972,6 +1210,8 @@ for row in DATA_GRAFANA_ROWS:
     all_panel_ids.append(row.id)
     all_panel_ids.extend(panel.id for panel in row.panels)
 
+all_panel_ids.sort()
+
 assert len(all_panel_ids) == len(
     set(all_panel_ids)
 ), f"Duplicated id found. Use unique id for each panel. {all_panel_ids}"
