@@ -22,6 +22,7 @@ from ray.data._internal.execution.interfaces import (
     RefBundle,
 )
 from ray.data._internal.execution.operators.input_data_buffer import InputDataBuffer
+from ray.data._internal.execution.progress_manager import RichExecutionProgressManager
 from ray.data._internal.execution.resource_manager import (
     ReservationOpResourceAllocator,
     ResourceManager,
@@ -43,13 +44,15 @@ from ray.data._internal.metadata_exporter import Topology as TopologyMetadata
 from ray.data._internal.progress_bar import ProgressBar
 from ray.data._internal.stats import DatasetStats, StatsManager, Timer
 from ray.data.context import OK_PREFIX, WARN_PREFIX, DataContext
+from ray.util.debug import log_once
 from ray.util.metrics import Gauge
 
 logger = logging.getLogger(__name__)
 
-# Force a progress bar update after this many events processed . This avoids the
-# progress bar seeming to stall for very large scale workloads.
+# Force a progress update after this many events processed. Avoids the
+# progress seeming to stall for very large scale workloads.
 PROGRESS_BAR_UPDATE_INTERVAL = 50
+PROGRESS_MANAGER_UPDATE_INTERVAL = 20
 
 # Interval for logging execution progress updates and operator metrics.
 DEBUG_LOG_INTERVAL_SECONDS = 5
@@ -76,6 +79,14 @@ class StreamingExecutor(Executor, threading.Thread):
         self._initial_stats: Optional[DatasetStats] = None
         self._final_stats: Optional[DatasetStats] = None
         self._global_info: Optional[ProgressBar] = None
+        self._progress_manager: Optional[RichExecutionProgressManager] = None
+
+        if not self._use_rich_progress() and log_once("rich_progress_disabled"):
+            logger.info(
+                "A new progress UI is available. To enable, set "
+                "`ray.data.DataContext.get_current()."
+                "enable_rich_progress_bars = True`."
+            )
 
         # The executor can be shutdown while still running.
         self._shutdown_lock = threading.RLock()
@@ -163,18 +174,26 @@ class StreamingExecutor(Executor, threading.Thread):
 
             logger.debug("Execution config: %s", self._options)
 
-            # Note: DAG must be initialized in order to query num_outputs_total.
-            # Note: Initialize global progress bar before building the streaming
-            # topology so bars are created in the same order as they should be
-            # displayed. This is done to ensure correct ordering within notebooks.
-            # TODO(zhilong): Implement num_output_rows_total for all
-            # AllToAllOperators
+        # Setup the streaming DAG topology and start the runner thread.
+        self._topology = build_streaming_topology(dag, self._options)
+
+        # Setup progress bars
+        if self._use_rich_progress():
+            self._progress_manager = RichExecutionProgressManager(
+                self._dataset_id, self._topology
+            )
+            self._progress_manager.start()
+        else:
             self._global_info = ProgressBar(
                 "Running", dag.num_output_rows_total(), unit="row"
             )
+            num_progress_bars = 1
+            for op_state in list(self._topology.values()):
+                if not isinstance(op_state.op, InputDataBuffer):
+                    num_progress_bars += op_state.initialize_progress_bars(
+                        num_progress_bars, self._options.verbose_progress
+                    )
 
-        # Setup the streaming DAG topology and start the runner thread.
-        self._topology, _ = build_streaming_topology(dag, self._options)
         self._resource_manager = ResourceManager(
             self._topology,
             self._options,
@@ -262,29 +281,33 @@ class StreamingExecutor(Executor, threading.Thread):
             self._sched_loop_duration_s.set(0, tags={"dataset": self._dataset_id})
             if self._data_context.enable_auto_log_stats:
                 logger.info(stats_summary_string)
-            # Close the progress bars from top to bottom to avoid them jumping
-            # around in the console after completion.
-            if self._global_info:
-                # Set the appropriate description that summarizes
-                # the result of dataset execution.
-                if exception is None:
-                    prog_bar_msg = (
-                        f"{OK_PREFIX} Dataset {self._dataset_id} execution finished in "
-                        f"{self._final_stats.time_total_s:.2f} seconds"
-                    )
-                else:
-                    prog_bar_msg = (
-                        f"{WARN_PREFIX} Dataset {self._dataset_id} execution failed"
-                    )
-                logger.info(prog_bar_msg)
-                self._global_info.set_description(prog_bar_msg)
+            # Close the progress manager with a finishing message.
+            if exception is None:
+                desc = (
+                    f"{OK_PREFIX} Dataset {self._dataset_id} execution finished in "
+                    f"{self._final_stats.time_total_s:.2f} seconds"
+                )
+            else:
+                desc = f"{WARN_PREFIX} Dataset {self._dataset_id} execution failed"
+
+            if self._use_rich_progress() and self._progress_manager:
+                self._progress_manager.close_with_finishing_description(
+                    desc, exception is None
+                )
+            elif not self._use_rich_progress() and self._global_info:
+                logger.info(desc)
+                self._global_info.set_description(desc)
                 self._global_info.close()
 
             timer = Timer()
 
             for op, state in self._topology.items():
                 op.shutdown(timer, force=force)
-                state.close_progress_bars()
+                if not self._use_rich_progress():
+                    # we only close sub-progress bars for the tqdm
+                    # implementation because closing is centrally
+                    # managed in the rich implementation.
+                    state.close_progress_bars()
 
             min_ = round(timer.min(), 3)
             max_ = round(timer.max(), 3)
@@ -465,15 +488,20 @@ class StreamingExecutor(Executor, threading.Thread):
             self._resource_manager.update_usages()
 
             i += 1
-            if i % PROGRESS_BAR_UPDATE_INTERVAL == 0:
+            if not self._use_rich_progress() and i % PROGRESS_BAR_UPDATE_INTERVAL == 0:
                 self._refresh_progress_bars(topology)
+            if self._use_rich_progress() and i % PROGRESS_MANAGER_UPDATE_INTERVAL == 0:
+                self._refresh_progress_manager(topology)
 
         # Trigger autoscaling
         self._cluster_autoscaler.try_trigger_scaling()
         self._actor_autoscaler.try_trigger_scaling()
 
         update_operator_states(topology)
-        self._refresh_progress_bars(topology)
+        if self._use_rich_progress():
+            self._refresh_progress_manager(topology)
+        else:
+            self._refresh_progress_bars(topology)
 
         self._update_stats_metrics(state=DatasetState.RUNNING.name)
         if time.time() - self._last_debug_log_time >= DEBUG_LOG_INTERVAL_SECONDS:
@@ -496,12 +524,23 @@ class StreamingExecutor(Executor, threading.Thread):
 
     def _refresh_progress_bars(self, topology: Topology):
         # Update the progress bar to reflect scheduling decisions.
+        assert not self._use_rich_progress()
         for op_state in topology.values():
             op_state.refresh_progress_bar(self._resource_manager)
         # Refresh the global progress bar to update elapsed time progress.
         if self._global_info:
             self._global_info.refresh()
 
+    def _refresh_progress_manager(self, topology: Topology):
+        # Update the progress manager to reflect scheduling decisions.
+        assert self._use_rich_progress()
+        if self._progress_manager:
+            for op_state in topology.values():
+                if not isinstance(op_state.op, InputDataBuffer):
+                    op_state.update_display_metrics(self._resource_manager)
+                    self._progress_manager.update_operator_progress(op_state)
+            self._progress_manager.refresh()
+
     def _consumer_idling(self) -> bool:
         """Returns whether the user thread is blocked on topology execution."""
         _, state = self._output_node
@@ -517,7 +556,7 @@ class StreamingExecutor(Executor, threading.Thread):
         pending_usage = self._resource_manager.get_global_pending_usage()
         limits = self._resource_manager.get_global_limits()
         resources_status = (
-            f"Running Dataset: {self._dataset_id}. Active & requested resources: "
+            f"Active & requested resources: "
             f"{running_usage.cpu:.4g}/{limits.cpu:.4g} CPU, "
         )
         if running_usage.gpu > 0:
@@ -538,7 +577,13 @@ class StreamingExecutor(Executor, threading.Thread):
             else:
                 pending_str = f"{pending_usage.gpu:.4g} GPU"
             resources_status += f" (pending: {pending_str})"
-        if self._global_info:
+
+        if self._use_rich_progress() and self._progress_manager:
+            self._progress_manager.update_resource_status(resources_status)
+        elif not self._use_rich_progress() and self._global_info:
+            resources_status = (
+                f"Running Dataset: {self._dataset_id}. {resources_status}"
+            )
             self._global_info.set_description(resources_status)
 
     def _get_operator_id(self, op: PhysicalOperator, topology_index: int) -> str:
@@ -584,6 +629,9 @@ class StreamingExecutor(Executor, threading.Thread):
             force_update=force_update,
         )
 
+    def _use_rich_progress(self):
+        return self._data_context.enable_rich_progress_bars
+
 
 def _validate_dag(dag: PhysicalOperator, limits: ExecutionResources) -> None:
     """Raises an exception on invalid DAGs.
@@ -645,9 +693,11 @@ def _debug_dump_topology(topology: Topology, resource_manager: ResourceManager)
     """
     logger.debug("Execution Progress:")
     for i, (op, state) in enumerate(topology.items()):
+        state.update_display_metrics(resource_manager)
         logger.debug(
             f"{i}: {state.summary_str(resource_manager)}, "
-            f"Blocks Outputted: {state.num_completed_tasks}/{op.num_outputs_total()}"
+            f"Blocks Outputted: {state.num_completed_tasks}/{op.num_outputs_total()}, "
+            f"Metrics: {state.op_display_metrics.display_str()}"
         )
 
 
@@ -680,10 +730,15 @@ class _ClosingIterator(OutputIterator):
             bundle = state.get_output_blocking(output_split_idx)
 
             # Update progress-bars
-            if self._executor._global_info:
+            using_rich = self._executor._use_rich_progress()
+            if not using_rich and self._executor._global_info:
                 self._executor._global_info.update(
                     bundle.num_rows(), op.num_output_rows_total()
                 )
+            elif using_rich and self._executor._progress_manager:
+                self._executor._progress_manager.update_total_progress(
+                    bundle.num_rows() or 0, op.num_output_rows_total()
+                )
 
             return bundle
 
