@@ -1,7 +1,8 @@
-from collections import deque
+import logging
 import queue
 import threading
 import time
+from collections import deque
 from typing import Any, Dict, Union
 
 import ray
@@ -22,8 +23,15 @@ from ray.rllib.utils.metrics import (
     NUM_ENV_STEPS_SAMPLED_LIFETIME,
 )
 from ray.rllib.utils.metrics.metrics_logger import MetricsLogger
+from ray.rllib.utils.metrics.ray_metrics import (
+    DEFAULT_HISTOGRAM_BOUNDARIES_SHORT_EVENTS,
+    TimerAndPrometheusLogger,
+)
 from ray.rllib.utils.schedules.scheduler import Scheduler
 from ray.rllib.utils.typing import ModuleID, ResultDict
+from ray.util.metrics import Gauge, Histogram
+
+logger = logging.getLogger(__name__)
 
 torch, _ = try_import_torch()
 
@@ -42,6 +50,51 @@ _CURRENT_GLOBAL_TIMESTEPS = None
 
 
 class IMPALALearner(Learner):
+    @override(Learner)
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        # Ray metrics
+        self._metrics_learner_impala_update = Histogram(
+            name="rllib_learner_impala_update_time",
+            description="Time spent in the 'IMPALALearner.update()' method.",
+            boundaries=DEFAULT_HISTOGRAM_BOUNDARIES_SHORT_EVENTS,
+            tag_keys=("rllib",),
+        )
+        self._metrics_learner_impala_update.set_default_tags(
+            {"rllib": self.__class__.__name__}
+        )
+
+        self._metrics_learner_impala_update_solve_refs = Histogram(
+            name="rllib_learner_impala_update_solve_refs_time",
+            description="Time spent on resolving refs in the 'Learner.update()'",
+            boundaries=DEFAULT_HISTOGRAM_BOUNDARIES_SHORT_EVENTS,
+            tag_keys=("rllib",),
+        )
+        self._metrics_learner_impala_update_solve_refs.set_default_tags(
+            {"rllib": self.__class__.__name__}
+        )
+
+        self._metrics_learner_impala_update_make_batch_if_necessary = Histogram(
+            name="rllib_learner_impala_update_make_batch_if_necessary_time",
+            description="Time spent on making a batch in the 'Learner.update()'.",
+            boundaries=DEFAULT_HISTOGRAM_BOUNDARIES_SHORT_EVENTS,
+            tag_keys=("rllib",),
+        )
+        self._metrics_learner_impala_update_make_batch_if_necessary.set_default_tags(
+            {"rllib": self.__class__.__name__}
+        )
+
+        self._metrics_learner_impala_get_learner_state_time = Histogram(
+            name="rllib_learner_impala_get_learner_state_time",
+            description="Time spent on get_state() in IMPALALearner.update().",
+            boundaries=DEFAULT_HISTOGRAM_BOUNDARIES_SHORT_EVENTS,
+            tag_keys=("rllib",),
+        )
+        self._metrics_learner_impala_get_learner_state_time.set_default_tags(
+            {"rllib": self.__class__.__name__}
+        )
+
     @override(Learner)
     def build(self) -> None:
         super().build()
@@ -137,56 +190,67 @@ class IMPALALearner(Learner):
         global _CURRENT_GLOBAL_TIMESTEPS
         _CURRENT_GLOBAL_TIMESTEPS = timesteps or {}
 
-        # Get the train batch from the object store.
-        training_data.solve_refs()
+        with TimerAndPrometheusLogger(self._metrics_learner_impala_update):
+            # Get the train batch from the object store.
+            with TimerAndPrometheusLogger(
+                self._metrics_learner_impala_update_solve_refs
+            ):
+                training_data.solve_refs()
 
-        batch = self._make_batch_if_necessary(training_data=training_data)
-        assert batch is not None
+            with TimerAndPrometheusLogger(
+                self._metrics_learner_impala_update_make_batch_if_necessary
+            ):
+                batch = self._make_batch_if_necessary(training_data=training_data)
+                assert batch is not None
 
-        if self.config.num_gpus_per_learner > 0:
-            self._gpu_loader_in_queue.put(batch)
-            self.metrics.log_value(
-                (ALL_MODULES, QUEUE_SIZE_GPU_LOADER_QUEUE),
-                self._gpu_loader_in_queue.qsize(),
-            )
-        else:
-            if isinstance(self._learner_thread_in_queue, CircularBuffer):
-                ts_dropped = self._learner_thread_in_queue.add(batch)
+            if self.config.num_gpus_per_learner > 0:
+                self._gpu_loader_in_queue.put(batch)
                 self.metrics.log_value(
-                    (ALL_MODULES, LEARNER_THREAD_ENV_STEPS_DROPPED),
-                    ts_dropped,
-                    reduce="sum",
+                    (ALL_MODULES, QUEUE_SIZE_GPU_LOADER_QUEUE),
+                    self._gpu_loader_in_queue.qsize(),
                 )
             else:
-                # Enqueue to Learner thread's in-queue.
-                _LearnerThread.enqueue(
-                    self._learner_thread_in_queue, batch, self.metrics
-                )
+                if isinstance(self._learner_thread_in_queue, CircularBuffer):
+                    ts_dropped = self._learner_thread_in_queue.add(batch)
+                    self.metrics.log_value(
+                        (ALL_MODULES, LEARNER_THREAD_ENV_STEPS_DROPPED),
+                        ts_dropped,
+                        reduce="sum",
+                    )
+                else:
+                    # Enqueue to Learner thread's in-queue.
+                    _LearnerThread.enqueue(
+                        self._learner_thread_in_queue, batch, self.metrics
+                    )
 
-        # TODO (sven): Find a better way to limit the number of (mostly) unnecessary
-        #  metrics reduces.
-        with self._num_updates_lock:
-            count = self._num_updates
-        result = {}
-        if count >= 20:
+            # TODO (sven): Find a better way to limit the number of (mostly) unnecessary
+            #  metrics reduces.
             with self._num_updates_lock:
-                self._num_updates = 0
-            result = self.metrics.reduce()
+                count = self._num_updates
+            result = {}
 
-        if return_state:
-            learner_state = self.get_state(
-                # Only return the state of those RLModules that are trainable.
-                components=[
-                    COMPONENT_RL_MODULE + "/" + mid
-                    for mid in self.module.keys()
-                    if self.should_module_be_updated(mid)
-                ],
-                inference_only=True,
-            )
-            learner_state[COMPONENT_RL_MODULE] = ray.put(
-                learner_state[COMPONENT_RL_MODULE]
-            )
-            result["_rl_module_state_after_update"] = learner_state
+            if count >= 20:
+                with self._num_updates_lock:
+                    self._num_updates = 0
+                result = self.metrics.reduce()
+
+            if return_state:
+                with TimerAndPrometheusLogger(
+                    self._metrics_learner_impala_get_learner_state_time
+                ):
+                    learner_state = self.get_state(
+                        # Only return the state of those RLModules that are trainable.
+                        components=[
+                            COMPONENT_RL_MODULE + "/" + mid
+                            for mid in self.module.keys()
+                            if self.should_module_be_updated(mid)
+                        ],
+                        inference_only=True,
+                    )
+                    learner_state[COMPONENT_RL_MODULE] = ray.put(
+                        learner_state[COMPONENT_RL_MODULE]
+                    )
+                    result["_rl_module_state_after_update"] = learner_state
 
         return result
 
@@ -263,18 +327,71 @@ class _GPULoaderThread(threading.Thread):
         self._device = device
         self.metrics = metrics_logger
 
+        self._metrics_impala_gpu_loader_thread_step_time = Histogram(
+            name="rllib_learner_impala_gpu_loader_thread_step_time",
+            description="Time taken in seconds for gpu loader thread _step.",
+            boundaries=DEFAULT_HISTOGRAM_BOUNDARIES_SHORT_EVENTS,
+            tag_keys=("rllib",),
+        )
+        self._metrics_impala_gpu_loader_thread_step_time.set_default_tags(
+            {"rllib": "IMPALA/GPULoaderThread"}
+        )
+
+        self._metrics_impala_gpu_loader_thread_step_in_queue_get_time = Histogram(
+            name="rllib_learner_impala_gpu_loader_thread_step_get_time",
+            description="Time taken in seconds for gpu loader thread _step _in_queue.get().",
+            boundaries=DEFAULT_HISTOGRAM_BOUNDARIES_SHORT_EVENTS,
+            tag_keys=("rllib",),
+        )
+        self._metrics_impala_gpu_loader_thread_step_in_queue_get_time.set_default_tags(
+            {"rllib": "IMPALA/GPULoaderThread"}
+        )
+
+        self._metrics_impala_gpu_loader_thread_step_load_to_gpu_time = Histogram(
+            name="rllib_learner_impala_gpu_loader_thread_step_load_to_gpu_time",
+            description="Time taken in seconds for GPU loader thread _step to load batch to GPU.",
+            boundaries=DEFAULT_HISTOGRAM_BOUNDARIES_SHORT_EVENTS,
+            tag_keys=("rllib",),
+        )
+        self._metrics_impala_gpu_loader_thread_step_load_to_gpu_time.set_default_tags(
+            {"rllib": "IMPALA/GPULoaderThread"}
+        )
+
+        self._metrics_impala_gpu_loader_thread_in_qsize_beginning_of_step = Gauge(
+            name="rllib_impala_gpu_loader_thread_in_qsize_beginning_of_step",
+            description="Size of the _GPULoaderThread in-queue size, at the beginning of the step.",
+            tag_keys=("rllib",),
+        )
+        self._metrics_impala_gpu_loader_thread_in_qsize_beginning_of_step.set_default_tags(
+            {"rllib": "IMPALA/GPULoaderThread"}
+        )
+
     def run(self) -> None:
         while True:
-            self._step()
+            with TimerAndPrometheusLogger(
+                self._metrics_impala_gpu_loader_thread_step_time
+            ):
+                self._step()
 
     def _step(self) -> None:
-        # Get a new batch from the data (inqueue).
+        self._metrics_impala_gpu_loader_thread_in_qsize_beginning_of_step.set(
+            value=self._in_queue.qsize()
+        )
+        # Get a new batch from the data (in-queue).
         with self.metrics.log_time((ALL_MODULES, GPU_LOADER_QUEUE_WAIT_TIMER)):
-            ma_batch_on_cpu = self._in_queue.get()
+            with TimerAndPrometheusLogger(
+                self._metrics_impala_gpu_loader_thread_step_in_queue_get_time
+            ):
+                ma_batch_on_cpu = self._in_queue.get()
 
         # Load the batch onto the GPU device.
         with self.metrics.log_time((ALL_MODULES, GPU_LOADER_LOAD_TO_GPU_TIMER)):
-            ma_batch_on_gpu = ma_batch_on_cpu.to_device(self._device, pin_memory=False)
+            with TimerAndPrometheusLogger(
+                self._metrics_impala_gpu_loader_thread_step_load_to_gpu_time
+            ):
+                ma_batch_on_gpu = ma_batch_on_cpu.to_device(
+                    self._device, pin_memory=False
+                )
 
         if isinstance(self._out_queue, CircularBuffer):
             ts_dropped = self._out_queue.add(ma_batch_on_gpu)
@@ -310,9 +427,31 @@ class _LearnerThread(threading.Thread):
         #  metrics each 20 updates" logic right now.
         # self._out_queue: deque = out_queue
 
+        # Ray metrics
+        self._metrics_learner_impala_thread_step = Histogram(
+            name="rllib_learner_impala_learner_thread_step_time",
+            description="Time taken in seconds for learner thread _step.",
+            boundaries=DEFAULT_HISTOGRAM_BOUNDARIES_SHORT_EVENTS,
+            tag_keys=("rllib",),
+        )
+        self._metrics_learner_impala_thread_step.set_default_tags(
+            {"rllib": "IMPALA/LearnerThread"}
+        )
+
+        self._metrics_learner_impala_thread_step_update = Histogram(
+            name="rllib_learner_impala_learner_thread_step_update_time",
+            description="Time taken in seconds for learner thread _step update.",
+            boundaries=DEFAULT_HISTOGRAM_BOUNDARIES_SHORT_EVENTS,
+            tag_keys=("rllib",),
+        )
+        self._metrics_learner_impala_thread_step_update.set_default_tags(
+            {"rllib": "IMPALA/LearnerThread"}
+        )
+
     def run(self) -> None:
         while not self.stopped:
-            self.step()
+            with TimerAndPrometheusLogger(self._metrics_learner_impala_thread_step):
+                self.step()
 
     def step(self):
         global _CURRENT_GLOBAL_TIMESTEPS
@@ -345,12 +484,15 @@ class _LearnerThread(threading.Thread):
             #  this thread has the information about the min minibatches necessary
             #  (due to different agents taking different steps in the env, e.g.
             #  MA-CartPole).
-            self._update_method(
-                self=self.learner,
-                training_data=TrainingData(batch=ma_batch_on_gpu),
-                timesteps=_CURRENT_GLOBAL_TIMESTEPS,
-                _no_metrics_reduce=True,
-            )
+            with TimerAndPrometheusLogger(
+                self._metrics_learner_impala_thread_step_update
+            ):
+                self._update_method(
+                    self=self.learner,
+                    training_data=TrainingData(batch=ma_batch_on_gpu),
+                    timesteps=_CURRENT_GLOBAL_TIMESTEPS,
+                    _no_metrics_reduce=True,
+                )
             # TODO (sven): Figure out a way to use a results queue instaad of the "reduce
             #  metrics each 20 updates" logic right now.
             # self._out_queue.append(results)
