@@ -1,13 +1,14 @@
+import logging
+import math
+import time
 from collections import defaultdict
 from functools import partial
-import math
-import logging
-import time
 from typing import Collection, DefaultDict, Dict, List, Optional, Union
 
 import gymnasium as gym
 
 import ray
+from ray._common.deprecation import Deprecated
 from ray.rllib.algorithms.algorithm_config import AlgorithmConfig
 from ray.rllib.callbacks.utils import make_callback
 from ray.rllib.core import (
@@ -22,13 +23,12 @@ from ray.rllib.env.env_context import EnvContext
 from ray.rllib.env.env_runner import EnvRunner, ENV_STEP_FAILURE
 from ray.rllib.env.multi_agent_env import MultiAgentEnv
 from ray.rllib.env.multi_agent_episode import MultiAgentEpisode
-from ray.rllib.env.vector.vector_multi_agent_env import VectorMultiAgentEnv
-from ray.rllib.env.vector.registration import make_vec
 from ray.rllib.env.utils import _gym_env_creator
+from ray.rllib.env.vector.registration import make_vec
+from ray.rllib.env.vector.vector_multi_agent_env import VectorMultiAgentEnv
 from ray.rllib.utils import force_list
 from ray.rllib.utils.annotations import override
 from ray.rllib.utils.checkpoints import Checkpointable
-from ray._common.deprecation import Deprecated
 from ray.rllib.utils.framework import get_device, try_import_torch
 from ray.rllib.utils.metrics import (
     ENV_TO_MODULE_CONNECTOR,
@@ -413,6 +413,12 @@ class MultiAgentEnvRunner(EnvRunner, Checkpointable):
                         truncateds=truncateds[env_index],
                         extra_model_outputs=extra_model_outputs,
                     )
+
+                    # Ray metrics
+                    self._log_env_steps(
+                        metric=self._metrics_num_env_steps_sampled, num_steps=1
+                    )
+
                     # Only increase ts when we actually stepped (not reset'd as a reset
                     # does not count as a timestep).
                     env_ts += self._increase_sampled_metrics(
