@@ -648,20 +648,20 @@ Windows powershell users need additional escaping:
     "--system-reserved-cpu",
     required=False,
     type=float,
-    help="The amount of cpu cores to reserve for ray system processes. Cores can be "
-    "fractional i.e. 0.5 means half a cpu core. "
-    "By default, the min of 20% and 1 core will be reserved."
-    "Must be >= 0.5 and < total number of available cores. "
-    "This option only works if --enable-resource-isolation is set.",
+    help="The number of cpu cores to reserve for ray system processes. "
+    "Cores can be fractional i.e. 1.5 means one and a half a cpu core. "
+    "By default, the value will be atleast 1 core, and at maximum 3 cores. The default value "
+    "is calculated using the formula min(3.0, max(1.0, 0.05 * num_cores_on_the_system)) "
+    "This option only works if --enable_resource_isolation is set.",
 )
 @click.option(
     "--system-reserved-memory",
     required=False,
     type=int,
     help="The amount of memory (in bytes) to reserve for ray system processes. "
-    "By default, the min of 10% and 25GB plus object_store_memory will be reserved. "
-    "Must be >= 100MB and system-reserved-memory + object-store-bytes < total available memory "
-    "This option only works if --enable-resource-isolation is set.",
+    "By default, the value will be atleast 500MB, and at most 10GB. The default value is  "
+    "calculated using the formula min(10GB, max(500MB, 0.10 * memory_available_on_the_system)) "
+    "This option only works if --enable_resource_isolation is set.",
 )
 @click.option(
     "--cgroup-path",
@@ -670,9 +670,9 @@ Windows powershell users need additional escaping:
     type=str,
     help="The path for the cgroup the raylet should use to enforce resource isolation. "
     "By default, the cgroup used for resource isolation will be /sys/fs/cgroup. "
-    "The raylet must have read/write permissions to this path. "
+    "The process starting ray must have read/write permissions to this path.  "
     "Cgroup memory and cpu controllers be enabled for this cgroup. "
-    "This option only works if --enable-resource-isolation is set.",
+    "This option only works if enable_resource_isolation is True.",
 )
 @add_click_logging_options
 @PublicAPI
