@@ -38,6 +38,9 @@ class ServeUsageTag(Enum):
     AUTO_NUM_REPLICAS_USED = TagKey.SERVE_AUTO_NUM_REPLICAS_USED
     CUSTOM_REQUEST_ROUTER_USED = TagKey.SERVE_CUSTOM_REQUEST_ROUTER_USED
     NUM_REPLICAS_VIA_API_CALL_UPDATED = TagKey.SERVE_NUM_REPLICAS_VIA_API_CALL_UPDATED
+    NUM_REPLICAS_USING_ASYNCHRONOUS_INFERENCE = (
+        TagKey.SERVE_NUM_REPLICAS_USING_ASYNCHRONOUS_INFERENCE
+    )
 
     def record(self, value: str):
         """Record telemetry value."""
