@@ -65,24 +65,6 @@ class ScalingConfig(ScalingConfigV1):
             auto-detected for TPUs and added as Ray node labels. This arg enables
             SPMD execution of the training workload. This field is required
             when `use_tpu` is True and `num_workers` is greater than 1.
-
-    Example:
-
-        .. testcode::
-
-            from ray.train import ScalingConfig
-            scaling_config = ScalingConfig(
-                # Number of distributed workers.
-                num_workers=2,
-                # Turn on/off GPU.
-                use_gpu=True,
-            )
-
-        .. testoutput::
-            :hide:
-
-            ...
-
     """
 
     trainer_resources: Optional[dict] = None
@@ -168,7 +150,6 @@ class FailureConfig(FailureConfigV1):
     controller_failure_limit: int = -1
 
     def __post_init__(self):
-        # TODO(justinvyu): Add link to migration guide.
         if self.fail_fast != _DEPRECATED:
             raise DeprecationWarning(FAIL_FAST_DEPRECATION_MESSAGE)
 
@@ -226,7 +207,6 @@ class RunConfig:
         if isinstance(self.storage_path, Path):
             self.storage_path = self.storage_path.as_posix()
 
-        # TODO(justinvyu): Add link to migration guide.
         run_config_deprecation_message = (
             "`RunConfig({})` is deprecated. This configuration was a "
             "Ray Tune API that did not support Ray Train usage well, "
@@ -264,6 +244,23 @@ class RunConfig:
                 "https://github.com/ray-project/ray/issues/49454"
             )
 
+        # TODO: Create a separate V2 CheckpointConfig class.
+        if not isinstance(self.checkpoint_config, CheckpointConfig):
+            raise ValueError(
+                f"Invalid `CheckpointConfig` type: {self.checkpoint_config.__class__}. "
+                "Use `ray.train.CheckpointConfig` instead. "
+                "See this issue for more context: "
+                "https://github.com/ray-project/ray/issues/49454"
+            )
+
+        if not isinstance(self.failure_config, FailureConfig):
+            raise ValueError(
+                f"Invalid `FailureConfig` type: {self.failure_config.__class__}. "
+                "Use `ray.train.FailureConfig` instead. "
+                "See this issue for more context: "
+                "https://github.com/ray-project/ray/issues/49454"
+            )
+
     @cached_property
     def storage_context(self) -> StorageContext:
         return StorageContext(
